
`default_nettype none

module top(
	input  wire Reset,				// Pin 17
	
	input wire Encoder1A_i,			// Pin 68
	input wire Encoder1B_i,			// Pin 67
	input wire Encoder1S_i,			// Pin 66
	input wire Encoder2A_i,			// Pin 71
	input wire Encoder2B_i,			// Pin 70
	input wire Encoder2S_i,			// Pin 69
	
	output wire LED1,				// Pin 12
	output wire LED2,				// Pin 13
	output wire [7:0] Cathodes_o,
	output wire [7:0] Segments_o
);
	
	// Generator sygnau zegarowego
	parameter CLOCK_HZ = 14_000_000;
	wire Clock;
	OSCH #(
		.NOM_FREQ("14.00")
	) OSCH_inst(
		.STDBY(1'b0),
		.OSC(Clock),
		.SEDSTDBY()
	);
	
	// Encoder 1
	wire Increment1;
	wire Decrement1;
	wire ButtonPress1;
	wire ButtonRelease1;
	
	Encoder Encoder1(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(Encoder1A_i),
		.AsyncB_i(Encoder1B_i),
		.AsyncS_i(Encoder1S_i),
		.Increment_o(Increment1),
		.Decrement_o(Decrement1),
		.ButtonPress_o(ButtonPress1),
		.ButtonRelease_o(ButtonRelease1),
		.ButtonState_o(LED1)
	);
	
	// Encoder 2
	wire Increment2;
	wire Decrement2;
	wire ButtonPress2;
	wire ButtonRelease2;
	
	Encoder Encoder2(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(Encoder2A_i),
		.AsyncB_i(Encoder2B_i),
		.AsyncS_i(Encoder2S_i),
		.Increment_o(Increment2),
		.Decrement_o(Decrement2),
		.ButtonPress_o(ButtonPress2),
		.ButtonRelease_o(ButtonRelease2),
		.ButtonState_o(LED2)
	);
	
	// Counter 1
	reg [15:0] Counter1;
	always @(posedge Clock, negedge Reset) begin
		if(!Reset)
			Counter1 <= 0;
		else if(ButtonPress1)
			Counter1 <= 16'h1234;
		else if(ButtonRelease1)
			Counter1 <= 16'h5678;
		else if(Increment1)
			Counter1 <= Counter1 + 1'b1;
		else if(Decrement1)
			Counter1 <= Counter1 - 1'b1;
	end
	
	// Counter 2
	reg [15:0] Counter2;
	always @(posedge Clock, negedge Reset) begin
		if(!Reset)
			Counter2 <= 0;
		else if(ButtonPress2)
			Counter2 <= 16'h1234;
		else if(ButtonRelease2)
			Counter2 <= 16'h5678;
		else if(Increment2)
			Counter2 <= Counter2 + 1'b1;
		else if(Decrement2)
			Counter2 <= Counter2 - 1'b1;
	end
	
	// Display instance
	DisplayMultiplex #(
		.CLOCK_HZ(CLOCK_HZ),
		.SWITCH_PERIOD_US(1000), // max 2000
		.DIGITS(8)
	) DisplayMultiplex_inst(
		.Clock(Clock),
		.Reset(Reset),
		.Data_i({Counter2, Counter1}),
		.DecimalPoints_i(8'b00010000),
		.Cathodes_o(Cathodes_o),
		.Segments_o(Segments_o),
		.SwitchCathode_o()
	);
	

endmodule

`default_nettype wire
